#ifndef CLASS_DEFINE
struct packet_handler
{
  struct lsh_object super;
  int (*(handler))(struct packet_handler *self, struct ssh_connection *connection, struct lsh_string *packet);
};
extern struct lsh_class packet_handler_class;
#endif /* !CLASS_DEFINE */

#ifndef CLASS_DECLARE
struct lsh_class packet_handler_class =
{ STATIC_HEADER,
  0, "packet_handler", sizeof(struct packet_handler),
  NULL,
  NULL
};
#endif /* !CLASS_DECLARE */

#ifndef CLASS_DEFINE
struct ssh_connection
{
  struct abstract_write super;
  struct lsh_string *client_version;
  struct lsh_string *server_version;
  struct lsh_string *session_id;
  struct resource_list *resources;
  UINT32 rec_max_packet;
  struct mac_instance *rec_mac;
  struct crypto_instance *rec_crypto;
  struct abstract_write *raw;
  struct abstract_write *write;
  struct mac_instance *send_mac;
  struct crypto_instance *send_crypto;
  int kex_state;
  struct kexinit *((kexinits)[2]);
  struct lsh_string *((literal_kexinits)[2]);
  struct newkeys_info *newkeys;
  struct packet_handler *((dispatch)[0x100]);
  struct packet_handler *ignore;
  struct packet_handler *unimplemented;
  struct packet_handler *fail;
};
extern struct lsh_class ssh_connection_class;
#endif /* !CLASS_DEFINE */

#ifndef CLASS_DECLARE
static void do_ssh_connection_mark(struct lsh_object *o, 
void (*mark)(struct lsh_object *o))
{
  struct ssh_connection *i = (struct ssh_connection *) o;
  mark((struct lsh_object *) i->resources);
  mark((struct lsh_object *) i->rec_mac);
  mark((struct lsh_object *) i->rec_crypto);
  mark((struct lsh_object *) i->raw);
  mark((struct lsh_object *) i->write);
  mark((struct lsh_object *) i->send_mac);
  mark((struct lsh_object *) i->send_crypto);
  {
  unsigned k;
  for (k=0; k<2; k++)
    mark((struct lsh_object *) (i->kexinits)[k]);
}
  mark((struct lsh_object *) i->newkeys);
  {
  unsigned k;
  for (k=0; k<0x100; k++)
    mark((struct lsh_object *) (i->dispatch)[k]);
}
  mark((struct lsh_object *) i->ignore);
  mark((struct lsh_object *) i->unimplemented);
  mark((struct lsh_object *) i->fail);
}

static void do_ssh_connection_free(struct lsh_object *o)
{
  struct ssh_connection *i = (struct ssh_connection *) o;
  lsh_string_free(i->client_version);
  lsh_string_free(i->server_version);
  lsh_string_free(i->session_id);
  {
  unsigned k;
  for (k=0; k<2; k++)
    lsh_string_free((i->literal_kexinits)[k]);
}
}

struct lsh_class ssh_connection_class =
{ STATIC_HEADER,
  &abstract_write_class, "ssh_connection", sizeof(struct ssh_connection),
  do_ssh_connection_mark,
  do_ssh_connection_free
};
#endif /* !CLASS_DECLARE */

